.mentions-input {
    position: relative;
}

.mentions-input .input {
    overflow: hidden;
    position: relative;
}

.mentions-input .highlighter {
    position: absolute;
    left: 0;
    right: 0;
    color: transparent;
    overflow: hidden;
    white-space: pre-wrap;
    word-wrap: break-word;

    border-style: solid;
    border-color: transparent;
}

.mentions-input .highlighter-content {
    position: relative;
}

.mentions-input .highlighter-content strong {
  font-weight: inherit;
  background-color: #a3bcea;
  display: inline;
  filter: progid:DXImageTransform.Microsoft.Alpha(opacity=0);
}

[data-mention] {
  background-color: #a3bcea;
  filter: progid:DXImageTransform.Microsoft.Alpha(opacity=0);
}