from setuptools import setup, find_packages

setup(
    name='django-mentions',
    version='0.1.8',
    author='Ivan Virabyan',
    packages=find_packages(),
    description='Facebook-like mentions for Django',
    include_package_data=True,
)
